/* -*- mode: c++ -*- */
// alineal.h

#ifndef _ALGL___
#define _ALGL___

#include <iostream>
#include <string>
#include <vector.h>
#include <matriz.h>
#include <permutacion.h>
#include <ccontrol.h>

/**
 *\defgroup P5 Documentacin de la Prctica 5 (lgebra Lineal y Diagonalizacin)
 */

/// Define una "mquina de lgebra lineal", 
/** Se trata de un objeto que encapsula las manipulaciones internas asociadas a un
 * problema de lgebra lineal, ofreciendo 
 * las soluciones al problema en la interfaz pblica
 *
 *\ingroup P5
 */
class ALineal {
private:
  /// (puntero a la) Matriz LU    
    RMatriz* pLU ;    
///  (puntero a la) Matriz inversa de A     
    RMatriz* pInv ;   
/// (puntero al) Vector de soluciones/incgnitas
    RVector* pV ;      
/// (puntero a la) Matriz de permutaciones
    Permutacion* pP ;  
/// determinante de A
    double pDet;       
/// dimensin de la matriz (= filas = cols)
    int pN ;           
/// nivel de impresin
    string pNivel;     
/// Cuadro de Control
    CControl pCC;       
    
  
    /// Constructor de copia privado para evitar que la clase pueda copiarse
    ALineal(const ALineal& m){};
    /// Operador de asignacin privado (la misma razn)
    ALineal& operator = (const ALineal& lu);

  /// descompone la matriz LU
    int descomponeLU(RMatriz& lu,Permutacion& p);

    /// resuelve el sistema lineal PLU*X = V
    int resuelveLU(const RMatriz& lu, const Permutacion& p,
			RVector& x);

public:
  /// Constructor 
    ALineal();
    /// Destructor
    ~ALineal();

    /// Establece una matriz para el sistema lineal
    /* (eMatriz = establece Matriz)
     */
    void eMatriz(const RMatriz& a);
  /// Establece un vector para el sistema lineal
    void eVector(const RVector& v);
    /// Establece el sistema lineal
    void eSistema(const RMatriz& a, const RVector& v);
  /// Establece el nivel de informacin
  /* posibles niveles: "detallado" y "silencioso"
   */
    void eInfo(string nivel="S");

    // Acciones del algoritmo
    ///  calcula/devuelve el vector de soluciones
    RVector x() ;     
///  calcula/devuelve la inversa de A
    RMatriz inversa();
  /// calcula/devuelve el determinante de A
    double det();      
/// calcula/devuelve la matriz LU compacta
    RMatriz lu();      
/// calcula/devuelve la matriz L
    RMatriz l() ;      
/// calcula/devuelve la matriz U
    RMatriz u() ;      
/// devuelve la matriz de permutaciones
    Permutacion p() const;   
};

#endif
