// linalg.cpp
// Implementa funciones en linalg.h
#include<linalg.h>
#include<error.h>
#include<iostream>

// Funciones de lgebra lineal

// Operaciones entre vectores y matrices

// m(i,j) * vt(j) = v(i)
RVector operator * (const RMatriz& m, const RVector& v)
{
    // Comprueba que tienen la dimensin adecuada 
    if (m.columnas() != v.dimension()) 
	error("El vector y la matriz tienen dimensiones incompatibles",
	      CEMALDIM);
  
    // Crea una nuevo vector con la dimensin adecuada
  RVector v1(m.filas());
  
  // Multiplica elemento a elemento
  for (int i = 1; i <= m.filas(); i++){
      v1(i) = 0;
      for (int j = 1; j <= m.columnas(); j++) 
	  v1(i)+= m(i, j) * v(j);
  }
  
  return v1;

}
// v(i)*m(i,j)  = v(j)
RVector operator * (const RVector& v, const RMatriz& m)
{
    // Comprueba que tienen la dimensin adecuada 
    if (m.filas() != v.dimension()) 
	error("El vector y la matriz tienen dimensiones incompatibles",
	      CEMALDIM);
  
    // Crea una nuevo vector con la dimensin adecuada
  RVector v1(m.columnas());
  
  // Multiplica elemento a elemento
  for (int i = 1; i <= m.columnas(); i++){
      v1(i) = 0;
      for (int j = 1; j <= m.filas(); j++) 
	  v1(i)+= m(i, j) * v(j);
  }
  
  return v1;
}

RVector proyectaFila(const RMatriz& a, int fila)
{
    RVector v(a.columnas());
    for (int i=1; i<= a.columnas(); i++)
	v(i) = a(fila,i);
    return v;
	
}
RVector proyectaColumna(const RMatriz& a, int col)
{
    RVector v(a.filas());
    for (int i=1; i<= a.filas(); i++)
	v(i) = a(i,col);
    return v;

}
void copiaFila(RMatriz& a, const RVector& v, int fila)
{
    for (int i=1; i<= a.columnas(); i++)
	a(fila,i) = v(i);
}

void copiaColumna(RMatriz& a, const RVector& v, int col)
{
    for (int i=1; i<= a.filas(); i++)
	a(i,col) = v(i);
}

// Operaciones con permutaciones

// Aplica la permutacin p al vector v
RVector operator * (const Permutacion& p, const RVector& v1){

    if (v1.dimension() != p.dimension())
	error("Dim de v debe ser igual a dim de p",CEMALDIM);

    // Crea un vector y copia v1
    RVector v = v1;

    for (int i = 1; i <= p.dimension(); i++){
	int k = p(i);
      
	while (k > i) {
	    k = p(k);
	}
	
	if (k < i)
	    continue ;
      
	// k == i, elemento ms pequeo en el ciclo

	int pk = p(k);
	if (pk == i)
	    continue ;
	
	// baraja los elementos del ciclo
	
        double t = v(i);
      
        while (pk != i){
	    v(k) = v(pk);
            k = pk;
            pk = p(k);

	}
	v(k) = t;

    }
    return v;
}

RMatriz operator * (const Permutacion& p, const RMatriz& m1){

    if (m1.filas() != p.dimension())
	error("filas de m debe ser igual a dim de p",CEMALDIM);

    // Crea una matriz y copia v1
    RMatriz m = m1;

    for (int i = 1; i <= p.dimension(); i++){
	int k = p(i);
      
	while (k > i) {
	    k = p(k);
	}
	
	if (k < i)
	    continue ;
      
	// k == i, elemento ms pequeo en el ciclo

	int pk = p(k);
	if (pk == i)
	    continue ;
	
	// baraja los elementos del ciclo

	// guarda la fila i en el vector t
	RVector t = proyectaFila(m, i);

        while (pk != i){
	    // mueve filas de acuerdo a la permutacin
	    m.copiaFila(k,pk);

            k = pk;
            pk = p(k);

	}
	// copia el vector t en la fila k de m
	copiaFila(m, t, k);

    }
    return m;
}
