/* -*- mode: c++ -*- */
// linalg.h
// Funciones de algebra lineal
// Manejan vectores, matrices y permutaciones
#ifndef _LINALG___
#define _LINALG___

#include <iostream>
#include <vector.h>
#include <matriz.h>
#include <permutacion.h>

//
// Operaciones entre vectores y matrices
//

/// m(i,j) * vt(j) = v(i)
/**
 *\ingroup P5
 */

RVector operator * (const RMatriz& m, const RVector& v);
/// v(i)*m(i,j)  = v(j)
/**
 *\ingroup P5
 */

RVector operator * (const RVector& v, const RMatriz& m);

/// proyecta filas de una matriz en un vector
/**
 *\ingroup P5
 */

RVector proyectaFila(const RMatriz& a, int fila);
/// proyecta columnas de una matriz en un vector
/**
 *\ingroup P5
 */
RVector proyectaColumna(const RMatriz& a, int col);

/// copia filas desde un vector en una matriz 
/**
 *\ingroup P5
 */

void copiaFila(RMatriz& a, const RVector& v, int fila);
/// copia columnas desde un vector en una matriz 
/**
 *\ingroup P5
 */

void copiaColumna(RMatriz& a, const RVector& v, int col);

//
// Operaciones con permutaciones, vectores y matrices
//
/// sobrecarga el producto de una permutacion y un vector
/**
 *\ingroup P5
 */

RVector operator * (const Permutacion& p, const RVector& v1);
/// sobrecarga el producto de una permutacin y una matriz
/**
 *\ingroup P5
 */

RMatriz operator * (const Permutacion& p, const RMatriz& m1);

#endif
