// matriz.cpp
// Implementacion de RMatriz

#include <error.h>
#include <matutil.h>
#include <matriz.h>
#include <smatriz.h>
#include <iomanip>
#include <fstream>

// Constructor por defecto
// Matriz de 1x1
RMatriz::RMatriz() 
{
    pFilas = 1;
    pCols = 1;
    pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);
    (*this).anulaElementos();
}

// Constructor estndar
// Crea una matriz con i filas y j columnas m(i,j)
RMatriz::RMatriz(int i, int j) 
{
    pFilas = i;
    pCols = j;
    if (i*j < 1)
	error("Error, la dimensin mnima de la matrix es 1",
	      CERANGO);
    // la dimensin de la matriz es ixj
    pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);
    (*this).anulaElementos();
}

// Constructor de copia
RMatriz::RMatriz(const RMatriz& m) 
{

    pFilas = m.pFilas;
    pCols = m.pCols;
    pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);
    
    // Copia cada pElem 
    for (int i = 1; i <= pFilas; i++)
      for (int j = 1; j <= pCols; j++)
	  (*this)(i, j) = m(i, j);
}

// Destructor
RMatriz::~RMatriz() 
{
  delete[] pElem;
}

// Devuelve nmero de filas
int  RMatriz::filas() const 
{
  return pFilas;
}
// Devuelve nmero de columnas
int  RMatriz::columnas() const 
{
  return pCols;
}

// Devuelve un elemento de la matriz
const double& RMatriz::operator () (int fila, int col) const 
{
    // Comprueba rango
    if (fila < 1 || fila > pFilas)
	error("Error, fila fuera de rango",CERANGO);
    if (col < 1 || col > pCols)
	error("Error, columna fuera de rango",CERANGO);
 
    return pElem[(col - 1) + pCols * (fila - 1)];
}

// Asigna un valor
double& RMatriz::operator () (int fila, int col)  
{
    // Comprueba rango
    if (fila < 1 || fila > pFilas)
	error("Error, fila fuera de rango",CERANGO);
    if (col < 1 || col > pCols)
	error("Error, columna fuera de rango",CERANGO);
 
    return pElem[(col - 1) + pCols * (fila - 1)];
}

//Operadores

// Operacin = ( m = m1)  
// Entre matrices reales
RMatriz& RMatriz::operator = (const RMatriz& m)
{
    // Libera la memoria
    delete pElem;
    pFilas = m.pFilas;
    pCols = m.pCols;
    pElem = new double[pFilas * pCols];
    if (!pElem)
	error("Imposible crear matriz",CENOMEM);

    // Copia cada pElem 
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) = m(i, j);

    return *this;
}


// Operacin += (m+=m1 -->m = m + m1)
RMatriz& RMatriz::operator += (const RMatriz& m) 
{
    // Comprueba dimensin
    if (m.pFilas != pFilas || m.pCols != pCols)
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);
  
    // Suma cada elemento
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) += m(i, j);
  
    return *this;
}

// Operacin -= (m-=m1 -->m = m - m1)
RMatriz& RMatriz::operator -= (const RMatriz& m) 
{
    // Comprueba dimensin
    if (m.pFilas != pFilas || m.pCols != pCols)
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);
  
    // Resta cada elemento
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) -= m(i, j);
  
    return *this;
}

// Operacin *= (m*=a -->m = m * a)
RMatriz& RMatriz::operator *= (double a) 
{
    // Multiplica cada elemento de la matriz por el factor a */ 
    for (int i = 1; i <= pFilas; i++)
	for (int j = 1; j <= pCols; j++)
	    (*this)(i, j) *= a;
    return *this;
}
RMatriz& RMatriz::intercambiaFilas(int i, int j){
    for (int k = 1; k<= pCols; k++){
	double temp = (*this)(i,k);
	(*this)(i,k) = (*this)(j,k);
	(*this)(j,k) = temp;
    }
    return *this;
}
RMatriz& RMatriz::anulaElementos(){
    for (int i = 1; i<= pFilas; i++)
	for (int j = 1; j<= pCols; j++)
	    (*this)(i,j) = 0.;

    return *this;
}
RMatriz& RMatriz::asignaIdentidad(){
    (*this).anulaElementos();
    for (int i = 1; i<= pFilas; i++){
	(*this)(i,i) = 1;
    }
    return *this;
}

RMatriz& RMatriz::intercambiaColumnas(int i, int j){
    for (int k = 1; k<= pFilas; k++){
	double temp = (*this)(k,i);
	(*this)(k,i) = (*this)(k,j);
	(*this)(k,j) = temp;
    }
    return *this;
}
RMatriz& RMatriz::copiaFila(int i, int j){
    for (int k = 1; k<= pCols; k++){
	(*this)(i,k) = (*this)(j,k);
    }
    return *this;
}
RMatriz& RMatriz::copiaColumna(int i, int j){
    for (int k = 1; k<= pFilas; k++){
	(*this)(k,i) = (*this)(k,j);
    }
    return *this;
}


// Lee una matriz de un fichero en disco
int RMatriz::lee(string fichero){
    // Intenta abrir el fichero para lectura
    ifstream fcin(fichero.c_str(),ios::in);
    if ( !fcin){
	error("Imposible abrir fichero de entrada en Matriz::lee",
	      CENOFCHRO);
	return CENOFCHRO;
    }
    // lee la matriz
    for (int i = 1; i <= (*this).filas(); i++) 
	for (int j = 1; j <=(*this).columnas(); j++)
	    fcin >> (*this)(i,j);
    return EXITO;
}

// escribe una matriz en un fichero en disco
int RMatriz::escribe(string fichero){
    // Intenta abrir el fichero para escritura
    ofstream fcout(fichero.c_str(),ios::out);
    if ( !fcout){
	error("Imposible abrir fichero salida en Matriz::escribe",
	      CENOFCHRO);
	return CENOFCHRO;
    }
    // escribe la matriz
    for (int i = 1; i <= (*this).filas(); i++){ 
	for (int j = 1; j <= (*this).columnas(); j++)
	    fcout << (*this)(i,j)<< " " ;
	fcout << endl;
    }
    return EXITO;
}

// Operaciones con matrices

// Cambio de signo
RMatriz operator - (const RMatriz& m1)
{
    // Crea una nueva matriz como una copia de m1
    RMatriz m = m1;   // Uso del operador de asignacin (=)
    for (int i = 1; i <= m.filas(); i++)
	for (int j = 1; j <= m.columnas(); j++)
	m(i,j) = -m1(i,j);   
    return m;
}

// Suma
RMatriz operator + (const RMatriz& m1, const RMatriz& m2) {
    // Comprueba dimensin
    if (m1.filas() != m2.filas() || 
	m1.columnas() != m2.columnas())
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);

    // Crea una nueva matriz copia de m1
    RMatriz m(m1);  // m = m1
    // Y suma
    m+=m2;          // m = m+m2 =m1+m2
    return m;
}

// Resta
RMatriz operator - (const RMatriz& m1, const RMatriz& m2) {
    // Comprueba dimensin
    if (m1.filas() != m2.filas() || 
	m1.columnas() != m2.columnas())
	error("Las matrices no tienen la misma dimensin",
	      CEMALDIM);

    // Crea una nueva matriz copia de m1
    RMatriz m(m1);  // m = m1
    // Y resta
    m-=m2;          // m = m+m2 =m1+m2
    return m;
}
// Producto por un escalar
RMatriz operator * (const RMatriz& m1, double a)
{
    // Crea una copia de m1
    RMatriz m(m1);   // Uso del operador de asignacin (=)
    m*=a;            // m *=a --> m = m*a
    return m;
}

RMatriz operator * (double a, const RMatriz& m)
{
    return m*a;  // a* v = v*a
}

// Producto de matrices
RMatriz operator * (const RMatriz& m1, const RMatriz& m2) 
{
    // Comprueba que tienen la dimensin adecuada 
    if (m1.columnas() != m2.filas()) 
	error("Intento de multiplicar matrices con dimensiones incompatibles",
	      CEMALDIM);
  
    // Crea una nueva matriz con la dimensin adecuada
  RMatriz m(m1.filas(), m2.columnas());
  
  // Multiplica elemento a elemento
  for (int i = 1; i <= m1.filas(); i++)
    for (int j = 1; j <= m2.columnas(); j++) {
	double subtotal = 0;
	for (int k = 1; k <= m1.columnas(); k++)
	    subtotal += m1(i, k) * m2(k, j);
      
	m(i, j) = subtotal;
    }
  
  return m;
}

// Matriz transpuesta
RMatriz traspuesta(const RMatriz& m) 
{
    // Crea una matriz de columnas x filas
    RMatriz mt(m.columnas(), m.filas());

    // Copia filas en columnas
    for (int i = 1; i <= mt.filas(); i++)
	for (int j = 1; j <= mt.columnas(); j++)
	    mt(i, j) = m(j, i);

    return mt;
}

// mximo y mnimo
double maximo(const RMatriz& m, int& imax, int& jmax)
{
    double max = 0;
    imax = 0;
    jmax = 0;
    for (int i=1; i<=m.filas();i++){
	for (int j=1; j<=m.columnas();j++){
	    if (abs(m(i,j)) > max){
		max = abs(m(i,j));
		imax = i;
		jmax = j;
	    }
	}
    }
    return max;    
}
double minimo(const RMatriz& m, int& imin, int& jmin)
{
    double min = BIG;
    imin = 0;
    jmin = 0;
    for (int i=1; i<=m.filas();i++){
	for (int j=1; j<=m.columnas();j++){
	    if (abs(m(i,j)) < min){
		min = abs(m(i,j));
		imin = i;
		jmin = j;
	    }
	}
    }
    return min;    
}
double maxNoDiag(const RMatriz& m, int& imax, int& jmax)
{
    double maximo = 0;
    imax = 0;
    jmax = 0;
    for (int i=1; i<=m.filas();i++){
	for (int j=1; j<=m.columnas();j++){
	    if (abs(m(i,j)) > maximo && (i != j)){
		maximo = abs(m(i,j));
		imax = i;
		jmax = j;
	    }
	}
    }
    return maximo;    
}

double minNoDiag(const RMatriz& m, int& imin, int& jmin)
{
    double minimo = BIG;
    imin = 0;
    jmin = 0;
    for (int i=1; i<=m.filas();i++){
	for (int j=1; j<=m.columnas();j++){
	    if (abs(m(i,j)) < minimo && (i != j)){
		minimo = abs(m(i,j));
		imin = i;
		jmin = j;
	    }
	}
    }
    return minimo;    
}

// Escribe una matriz
ostream& operator << (ostream& s, const RMatriz& m) 
{
    s << endl;
    for (int i = 1; i <= m.filas(); i++) {
	s << "| ";
    
	for (int j = 1; j <= m.columnas(); j++){
	    if (abs(m(i,j)) < SMALL) 
		s << setw(8)<<setprecision(2)<< 0 << "\t";
	    else 
		s << setw(8)<<setprecision(2)<<m(i, j) << "\t";
	}
	s << " |" << endl;;
    }
    return s;
}
