/* -*- mode: c++ -*- */
// matriz.h
// Clases de matrices

#ifndef _MATRIZ_
#define _MATRIZ_

#include <iostream>
#include <string>

using namespace std;

// Class RMatriz
/// Clase de matrices reales (ampliada con respecto a prctica 4)
/**
 *\ingroup P5
 */

class RMatriz {
 private:
/// arreglo dinmico de elementos
  double* pElem;         
/// nmero de filas 
  int pFilas;           
/// nmero de columnas 
  int pCols;            

 public:
    /// Constructor por defecto (matriz de dimensin 1 x 1)
    RMatriz();
    /// Constructor (matriz de dimensin n x m)
    RMatriz(int nfilas, int ncols);
  /// Constructor de copia
    RMatriz(const RMatriz& m);
    /// Destructor
    ~RMatriz();

    // Funciones de acceso
/// devuelve el nmero de filas
    int filas() const;    
  /// devuelve el nmero de columnas  
    int columnas() const; 

    // Operadores
    /// lectura (ejemplo, cout << a(i,j))
    const double& operator () (int fila, int col) const;
  /// asignacin (ejemplo,  a(i,j) = 7.)
    double& operator () (int fila, int col);

  /// Operador = entre matrices reales    
    RMatriz& operator = (const RMatriz& m);
  /// m1+=m2
    RMatriz& operator += (const RMatriz& m);
  /// m1-=m2
    RMatriz& operator -= (const RMatriz& m);
  /// m*=d
    RMatriz& operator *= (double a);

    // otras funciones
    /// intercambia fila i <--> fila j
    RMatriz& intercambiaFilas(int i, int j);
    /// intercambia columna i <--> columna j
    RMatriz& intercambiaColumnas(int i, int j);
  /// copia fila j en fila i
    RMatriz& copiaFila(int i, int j);
  /// copia columna j en columna i
    RMatriz& copiaColumna(int i, int j);
  /// anula todos los elementos de la matriz
    RMatriz& anulaElementos();
  /// asigna la matriz identidad
    RMatriz& asignaIdentidad();
  /// lee una matriz de un fichero
    int lee(string fichero);
  /// escribe una matriz en un fichero
    int escribe(string fichero);
};


/* Operaciones con matrices */

/// -m
RMatriz operator - (const RMatriz& m);
/// m1 + m2
RMatriz operator + (const RMatriz& m1, const RMatriz& m2);
/// m1 -m2
RMatriz operator - (const RMatriz& m1, const RMatriz& m2);
/// m1 *m2
RMatriz operator * (const RMatriz& m1, const RMatriz& m2);
/// m1 *d
RMatriz operator * (const RMatriz& v1, double d);
/// d*m1
RMatriz operator * (double d, const RMatriz& v1);
/// traspuesta
RMatriz traspuesta(const RMatriz& m);
/// devuelve el mximo elemento en valor absoluto
double maximo(const RMatriz& m, int& i, int& j);
/// devuelve el mnimo elemento en valor absoluto
double minimo(const RMatriz& m, int& i, int& j);
/// devuelve el mximo elemento en valor absoluto fuera de la diagonal
double maxNoDiag(const RMatriz& m, int& i, int& j);
/// devuelve el mnimo elemento en valor absoluto fuera de la diagonal
double minNoDiag(const RMatriz& m, int& i, int& j);
/// escribe una matriz
ostream& operator << (ostream& s, const RMatriz& m);

#endif   /* _MATRIZ_ */
