// permutacion.cpp
// Implementacin de la clase permutacion

#include <error.h>
#include <permutacion.h>

// Constructor por defecto. Crea la permutacin unidad
// del conjunto de un elemento

Permutacion::Permutacion() 
{
    pDim = 1;
    pSigno = 1;
    pElem = new int[pDim]; 
    if (!pElem)
	error("Imposible crear permutacin",CENOMEM);

    (*this).ini();

}

// Constructor estndar. Crea una permutacion de dimensin n
// y la inicializa a la permutacion unidad (1,2...n)
Permutacion::Permutacion(int n) 
{
    if (n < 1)
	error("La dimensin mnima de la permutacin es 1",CERANGO);
    pDim = n;
    pSigno = 1;
    pElem = new int[pDim]; 
    if (!pElem)
	error("Imposible crear permutacin",CENOMEM);

    (*this).ini();

}
// Constructor de copia
Permutacion::Permutacion(const Permutacion& pt) 
{

    // Crea una nueva permutacin con la dimensin de pt  
    pDim = pt.pDim;
    pSigno = pt.pSigno;
    pElem = new int[pDim];
    if (!pElem)
	error("Imposible crear permutacin",CENOMEM);
    
    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
	(*this)(i) = pt(i);
}

// Destructor
Permutacion::~Permutacion() 
{
    delete[] pElem;
}

// Funciones de acceso

// devuelve la dimensin de la permutacin
int Permutacion::dimension() const
{
    return pDim;
}

// signo 
int Permutacion::signo() const
{
    return pSigno;
}

// Operadores

// Devuelve el elemento v(i), i=1,2...n
const int& Permutacion::operator () (int i) const
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}
// Asigna el elemento v(i), i=1,2...n
int& Permutacion::operator () (int i)
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}

// intercambia elemento i y j
Permutacion& Permutacion::intercambia(int i, int j)
{
    
    if (i > pDim)
	error("Primer ndice fuera de rango",CEINVAL);
    
    if (j > pDim)
	error("Segundo ndice fuera de rango",CEINVAL);

    if (i != j){
	int tmp = (*this)(i);
	(*this)(i) = (*this)(j);
	(*this)(j) = tmp;
    }
    pSigno = -pSigno;
    return *this;    
}

// incializa la permutacion a la unidad
Permutacion& Permutacion::ini(){
    for (int i=1; i <= (*this).dimension(); i++)
	(*this)(i) = i;
    pSigno = 1;

    return *this;
}

// Permutacin nula (final de la cadena lexicogrfica)
Permutacion& Permutacion::anula(){
    for (int i=1; i <= (*this).dimension(); i++)
	(*this)(i) = 0;
    pSigno = 0;
    return *this;
}

// Operacin = ( v = v1)  
Permutacion& Permutacion::operator = (const Permutacion& v1)
{
    // Libera la memoria
    delete pElem;
    // Crea una nueva permutacin con la dimensin de v1  
    pDim = v1.pDim;
    pSigno = v1.pSigno;
    pElem = new int[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);
    
    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
	(*this)(i) = v1(i);
    return *this;
}

Permutacion& Permutacion::siguiente()
{
    // Reemplaza p con la siguiente permutacin en orden
    // lexicogrfico. Devuelve NULO (0,0,0...0) cuando no hay
    // ms permutaciones

    Permutacion& p = *this;  // simplifica notacin

    if (p.dimension() < 2) 
	return p.anula();

    int i = p.dimension() - 1;

    while ((p(i) > p(i+1) && (i != 1)))
	i--;
    
    if ((i == 1) && (p(1) > p(2)))  
	return p.anula();  // no hay permutacin siguiente

    int k = i + 1;

    for (int j = i + 1; j <= p.dimension(); j++ ){
	if ((p(j) > p(i)) && (p(j) < p(k)))
	    k = j;
    }
  /* intercambia i y k */

    int tmp = p(i);
    p(i) = p(k);
    p(k) = tmp;

    int top = (p.dimension() + i + 1) ;

    for (int j = i + 1; j <= (top/2); j++){
	int tmp = p(j);
	p(j) = p(top - j);
	p(top - j) = tmp;
    }

    return p;
}

Permutacion& Permutacion::previa()
{
    // Reemplaza p con la permutacin previa en orden
    // lexicogrfico. Al final del ciclo vuelve a la permutacin original

    Permutacion& p = *this;  // simplifica notacin

    if (p.dimension() < 2) 
	return p.anula();  // no hay permutacin siguiente

    int i = p.dimension() - 1;
    
    while ((p(i) < p(i+1) && (i != 1)))
	i--;
    
    if ((i == 1) && (p(1) < p(2)))  // no hay permutacin previa
	return p.anula();  // no hay permutacin siguiente
    
    int k = i + 1;

    for (int j = i + 1; j <= p.dimension(); j++ ){
	if ((p(j) < p(i)) && (p(j) > p(k)))
	    k = j;
    }

  /* intercambia i y k */

    int tmp = p(i);
    p(i) = p(k);
    p(k) = tmp;

    int top = (p.dimension() + i + 1) ;

    for (int j = i + 1; j <= (top / 2); j++){
	int tmp = p(j);
	p(j) = p(top - j);
	p(top - j) = tmp;
    }

    return p;
}

bool Permutacion::operator == (const Permutacion& p1)
{   
    Permutacion& p = *this;  // simplifica notacin
    
    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    bool test = true;
    for (int i=1; i<= p1.dimension(); i++){
	if (p1(i) != p(i)) {
	    test = false;
	    break;
	}
    }
    return test;
}

bool Permutacion::operator != (const Permutacion& p1)
{   
    Permutacion& p = *this;  // simplifica notacin

    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    bool test = false;
    for (int i=1; i<= p1.dimension(); i++){
	if (p1(i) != p(i)) {
	    test = true;
	    break;
	}
    }
    return test;    
}

bool Permutacion::operator > (const Permutacion& p1)
{    
    Permutacion& p = *this;  // simplifica notacin

    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    bool test = true;
    // Permutacin nula

    while (p.siguiente() !=  pnula(p.dimension())){	
	if (p == p1) test = false;
    }
    return test;
}

bool Permutacion::operator >= (const Permutacion& p1)
{
    Permutacion& p = *this;  // simplifica notacin

    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    if (p > p1) return true;
    else if (p == p1) return true;
    else return false;
}
bool Permutacion::operator < (const Permutacion& p1)
{
    Permutacion& p = *this;  // simplifica notacin

    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    bool test = true;
    // Permutacin nula

    while (p.previa() !=  pnula(p.dimension())){	
	if (p == p1) test = false;
    }
    return test;

}
bool Permutacion::operator <= (const Permutacion& p1)
{
    Permutacion& p = *this;  // simplifica notacin

    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    if (p < p1) return true;
    else if (p == p1) return true;
    else return false;
}

// Operaciones con permutaciones
// Composicin 
Permutacion operator * (const Permutacion& p, const Permutacion& p1)
{
    if (p1.dimension() != p.dimension())
	error( "Las permutaciones deben tener la misma dimensin",
	       CEMALDIM);

    Permutacion v = p1;

    for (int i = 1; i <= p.dimension(); i++){
	int k = p(i);
      
	while (k > i) {
	    k = p(k);
	}
	
	if (k < i)
	    continue ;
      
	// k == i, elemento ms pequeo en el ciclo

	int pk = p(k);
	if (pk == i)
	    continue ;
	
	// baraja los elementos del ciclo
	
        int t = v(i);
      
        while (pk != i){
	    v(k) = v(pk);
            k = pk;
            pk = p(k);

	}
	v(k) = t;
    }

    return v;
}
// Permutacin inversa
Permutacion pinv(const Permutacion& p)
{
    Permutacion pi(p.dimension());
  
    for (int i = 1; i <= p.dimension(); i++) 
	pi(p(i)) = i ;

    return pi;    
}

// Permutacin nula (final de la cadena lexicogrfica)
Permutacion pnula(int n){
    Permutacion p(n);
    for (int i=1; i <= n; i++)
	p(i) = 0;
    return p;
}

ostream& operator << (ostream& s, const Permutacion& v){

    s << "( ";
    for (int i = 1; i <= v.dimension(); i++) 
	s << v(i) << " ";
    
    s << " )" ;
    return s;
}
