// permutacion.h
// Clases de permutaciones

#ifndef _PERMUT___
#define _PERMUT___

#include <iostream>

using namespace std;

// Clase Permutacion
///Representa una permutacin algebraica
/**
 *\ingroup P5
 */
class Permutacion {
 private:
/// arreglo dinmico de elementos
    int* pElem;      
/// dimensin de la permutacin
    int pDim;        
///  signo de la permutacin
    int pSigno;      
// devuelve permutacin nula
    Permutacion& anula();  
 public:
    /// Constructor por defecto (permutacin unidad conjunto 1 elemento)
    Permutacion();
    /// Constructor estndar 
    Permutacion(int dimension);
    /// Constructor de copia
    Permutacion(const Permutacion& m);
    /// Destructor
    ~Permutacion();

    // Funciones de acceso
/// dimensin de la permutacin
    int dimension() const;
/// signo de la permutacin  
    int signo() const;  
/// permutacin unidad
    Permutacion& ini();  
/// cambia la fila i por la j
    Permutacion& intercambia(int i, int j);  
/// siguiente permutacin en orden lexicogrfico
    Permutacion& siguiente();
/// previa permutacin en orden lexicogrfico  
    Permutacion& previa();
    /// igualdad lgica  
    bool operator == (const Permutacion& p1);
    /// desigualdad lgica  
    bool operator != (const Permutacion& p1);
    /// p1 > p2 (orden lexicogrfico )
    bool operator > (const Permutacion& p1);
    /// p1 >= p2 (orden lexicogrfico )
    bool operator >= (const Permutacion& p1);
    /// p1 < p2 (orden lexicogrfico )
    bool operator < (const Permutacion& p1);
    /// p1 <= p2 (orden lexicogrfico )
    bool operator <= (const Permutacion& p1);

    // Operadores
    /// p(i) lectura
    const int& operator () (int indice) const;
    /// p(i) escritura
    int& operator () (int indice);
    /// asigna una permutacin a otra
    Permutacion& operator = (const Permutacion& v);
};

// Operaciones con Permutaciones
/// Inversa
Permutacion pinv(const Permutacion& v);
/// Permutacin nula (final de la cadena lexicogrfica)
Permutacion pnula(int dimension);
/// Composicin de permutaciones
Permutacion operator * (const Permutacion& p1, const Permutacion& p2);
/// Escribe una permutacin por la pantalla
ostream& operator << (ostream& s, const Permutacion& v);

#endif   /* _VECTOR___ */
