/* -*- mode: c++ -*- */
// Implementacin de la clase SMatriz
// Matrices reales y simtricas

#include <smatriz.h>
#include <error.h>
#include <matutil.h>
#include <iomanip>

// Class SMatriz
// Define una clase de matrices reales y simtricas

// Constructor por defecto: Matriz 1 x 1
SMatriz::SMatriz(){
    pM = new RMatriz();
    pDim = 1;
}

// Constructor estndar: Toma la parte
// triangular superior de la matriz m
SMatriz::SMatriz(const RMatriz& m)
{
    // La matriz debe ser cuadrada
    if (m.filas() != m.columnas())
	error("SMatriz: requiere una matriz cuadrada", 
	      CENOCUAD);

    pDim = m.filas();    // dimensin
    pM = new RMatriz(m); // puntero a una matriz real copia de m
    RMatriz& rm = *pM;  //  rm refiere al valor de pM

    for (int i = 1; i<= pDim; i++){
	for (int j = i; j<= pDim; j++){
	    rm(i,j) = m(i,j); 
	    rm(j,i) = rm(i,j); 
	}
    }
}

// Constructor de copia
SMatriz::SMatriz(const SMatriz& sm)
{
    RMatriz& rsm = *(sm.pM); //matrix real interna a sm

    // puntero a una nueva matriz
    pM = new RMatriz(rsm);
    RMatriz& rm = *pM;  // matriz real interna al objeto
    pDim = sm.pDim;     // dimensin

    for (int i = 1; i<= pDim; i++){
	for (int j = i; j<= pDim; j++){
	    rm(i,j) = rsm(i,j); 
	    rm(j,i) = rm(i,j); 
	}
    }
}    

// Destructor
SMatriz::~SMatriz()
{
    delete pM;
}

// devuelve la dimensin de la matriz
int SMatriz::dimension() const
{
    return pDim;
}

// devuelve la matriz
const RMatriz& SMatriz::rmatriz() const
{
    return *pM;
}
    
// Operadores
// lectura 
const double& SMatriz::operator () (int i, int j) const
{ 
    RMatriz& m = *pM;  // notacin
    return m(i,j);
}
// asignacin (privado) 
double& SMatriz::operator () (int i, int j) 
{ 
    RMatriz& m = *pM;  // notacin
    return m(i,j);
}
    
// Operador = entre dos Smatrices
SMatriz& SMatriz::operator = (const SMatriz& m)
{
    delete pM;
    RMatriz& a = *(m.pM);
    SMatriz& sm = *this;  

    pM = new RMatriz(a);
    for (int i = 1; i<= pDim; i++){
	for (int j = 1; j<= pDim; j++){
	    sm(i,j) = a(i,j); 
	    sm(j,i) = a(i,j); 
	}
    }
    return sm;
}
    
// Operador = entre dos RMatriz y SMatriz
SMatriz& SMatriz::operator = (const RMatriz& a)
{
    delete pM;
    SMatriz& sm = *this;  

    pM = new RMatriz(a);
    for (int i = 1; i<= pDim; i++){
	for (int j = 1; j<= pDim; j++){
	    sm(i,j) = a(i,j); 
	    sm(j,i) = a(i,j); 
	}
    }
    return sm;
}
SMatriz& SMatriz::operator += (const SMatriz& sa)
{
    RMatriz& a = *(sa.pM); // referencia a la matriz en sa 
    RMatriz& m = *(this->pM); // referencia a la matriz en sm 
    SMatriz& sm = *this;  //  matriz simtrica que llama al operador
    
    m +=a; //Operador += definido para las Rmatrices a y m
    return sm;
}
SMatriz& SMatriz::operator -= (const SMatriz& sa)
{
    RMatriz& a = *(sa.pM); // referencia a la matriz en sa 
    RMatriz& m = *(this->pM); // referencia a la matriz en sm 
    SMatriz& sm = *this;  //  matriz simtrica que llama al operador
    
    m -=a; //Operador -= definido para las Rmatrices a y m
    return sm;
}
SMatriz& SMatriz::operator *= (double d)
{
    RMatriz& m = *(this->pM); // referencia a la matriz en sm 
    SMatriz& sm = *this;  

    m *=d;
    return sm;
}

// otras funciones
SMatriz& SMatriz::anulaElementos()
{
    RMatriz& m = *(this->pM); // referencia a la matriz en sm 
    SMatriz& sm = *this;  

    m.anulaElementos();
    return sm;

}
SMatriz& SMatriz::asignaIdentidad()
{
    RMatriz& m = *(this->pM); // referencia a la matriz en sm 
    SMatriz& sm = *this;  

    m.asignaIdentidad();
    return sm;
}

/* Operaciones con matrices */

SMatriz operator - (const SMatriz& sm)
{
    // copia la matriz real interna a sm 
    RMatriz rsm = sm.rmatriz();
    // Cambia el signo de rsm
    rsm = -rsm;
    SMatriz m(rsm);  // Crea una matriz simtrica a partir de rsm
    return m;
}
SMatriz operator + (const SMatriz& sm1, const SMatriz& sm2)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm1 = sm1.rmatriz();
    // copia la matriz real interna a sm2
    RMatriz rsm2 = sm2.rmatriz();
    // Sumalas
    RMatriz rm = rsm1 + rsm2;

    SMatriz m(rm);  // Crea una matriz simtrica a partir de rm
    return m;
}
SMatriz operator - (const SMatriz& sm1, const SMatriz& sm2)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm1 = sm1.rmatriz();
    // copia la matriz real interna a sm2
    RMatriz rsm2 = sm2.rmatriz();
    // Restalas
    RMatriz rm = rsm1 - rsm2;

    SMatriz m(rm);  // Crea una matriz simtrica a partir de rm
    return m;

}
SMatriz operator * (const SMatriz& sm1, double d)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    rsm *= d;        // multiplica por escalar
    SMatriz m(rsm);  
    return m;
}
SMatriz operator * (double d, const SMatriz& sm1)
{
    return sm1*d;
}

// Operacin * (entre dos matrices simtricas)
RMatriz operator * (const SMatriz& m1, const SMatriz& m2)
{
    // devuelve la matriz real interna a m1
    RMatriz r1 = m1.rmatriz();
    // devuelve la matriz real interna a m2
    RMatriz r2 = m2.rmatriz();
    return r1*r2;
}

SMatriz traspuesta(const SMatriz& sm1)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    // Traspone
    RMatriz rm = traspuesta(rsm); 
    SMatriz m(rm);  // Crea una copia (sm) de m1
    return m;
}

// mximo y mnimo
double maximo(const SMatriz& sm1, int& imax, int& jmax)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    return maximo(rsm,imax,jmax);  // maximo de la matriz interna
}
double minimo(const SMatriz& sm1, int& imin, int& jmin)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    return minimo(rsm,imin,jmin);  // maximo de la matriz interna
}
double maxNoDiag(const SMatriz& sm1, int& imax, int& jmax)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    return maxNoDiag(rsm,imax,jmax);  // maximo de la matriz interna
}

double minNoDiag(const SMatriz& sm1, int& imin, int& jmin)
{
    // copia la matriz real interna a sm1 
    RMatriz rsm = sm1.rmatriz();
    return minNoDiag(rsm,imin,jmin);  // maximo de la matriz interna
}

ostream& operator << (ostream& s, const SMatriz& sm1)
{
    RMatriz rsm = sm1.rmatriz();

    s << endl;
    for (int i = 1; i <= rsm.filas(); i++) {
	s << "|\t";
    
	for (int j = 1; j <= rsm.columnas(); j++){
	    if (abs(rsm(i,j)) < SMALL) 
		s << setw(10)<<setprecision(2)<< 0 << "\t";
	    else 
		s << setw(10)<<setprecision(2)<<rsm(i, j) << "\t";
	}
	s << "|" << endl;;
    }
    return s;
}

