/* -*- mode: c++ -*- */
// smatriz.h
// Matrices simtricas

#ifndef _SMATRIZ_
#define _SMATRIZ_

#include <matriz.h>

using namespace std;

// Class SMatriz
/// Define una clase de matrices reales y simtricas
/**
 *\ingroup P5
 */
class SMatriz {
private:
/// matriz real (cuadrada)
    RMatriz* pM;  
  /// dimensin de pM
    int pDim;     
    
    /// operador de asignacin privado
    double& operator () (int fila, int col) ;
public:
    /// Constructor por defecto (matriz simtrica 1x1)
    SMatriz();
  ///Constructor
  /** Una matriz simtrica slo puede construirse
   * a partir de una matriz normal (cuadrada)
   * de la cual tomamos su parte triangular superior
   */
    SMatriz(const RMatriz& m);
    /// Constructor de copia
    SMatriz(const SMatriz& m);
  /// Destructor
    ~SMatriz();

    /// Dimensin de la matriz
    int dimension() const;
  /// devuelve la matriz  
    const RMatriz& rmatriz() const;

    // Operadores
    /// lectura (ejemplo, cout << a(i,j))
    const double& operator () (int fila, int col) const;
    // No hay operador de asignacin pblico

    /// Operador = entre matrices simtricas    
    SMatriz& operator = (const SMatriz& m);
  /// Operador = entre una matriz real y una matriz simtrica    
    SMatriz& operator = (const RMatriz& m);
  /// s1+=s2
    SMatriz& operator += (const SMatriz& m);
  /// s1-=s2
    SMatriz& operator -= (const SMatriz& m);
  /// s1*=d
    SMatriz& operator *= (double a);

    // otras funciones
  /// anula elementos
    SMatriz& anulaElementos();
  /// asigna identidad
    SMatriz& asignaIdentidad();

};


/* Operaciones con matrices */
/// -s
SMatriz operator - (const SMatriz& m);
/// s1+s2
SMatriz operator + (const SMatriz& m1, const SMatriz& m2);
/// s1-s2
SMatriz operator - (const SMatriz& m1, const SMatriz& m2);
/// s*d
SMatriz operator * (const SMatriz& v1, double d);
/// d*s
SMatriz operator * (double d, const SMatriz& v1);
/// s1*s2
RMatriz operator * (const SMatriz& m1, const SMatriz& m2);
/// traspuesta
SMatriz traspuesta(const SMatriz& m);
/// mximo
double maximo(const SMatriz& m, int& i, int& j);
/// mnimo
double minimo(const SMatriz& m, int& i, int& j);
/// mximo no diagonal
double maxNoDiag(const SMatriz& m, int& i, int& j);
/// mnimo no diagonal
double minNoDiag(const SMatriz& m, int& i, int& j);
/// escribe por pantalla
ostream& operator << (ostream& s, const SMatriz& m);

#endif   /* _MATRIZ_ */
