// vector.cpp
// Implementacin de la clase RVector

#include <error.h>
#include <vector.h>
#include <matutil.h>
#include <fstream>
#include <iomanip>

// Constructor por defecto. Vector de dimensin 1
RVector::RVector() 
{
    pDim = 1;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);
    (*this).anulaElementos();
}
// Constructor estndar. Crea un vector de dimensin n
RVector::RVector(int n) 
{
    if (n < 1)
	error("Error, la dimensin mnima del vector es 1",CERANGO);
    pDim = n;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);
    (*this).anulaElementos();
}
// Constructor de copia
RVector::RVector(const RVector& v) 
{
    // Crea un nuevo vector con la dimensin de v  
    pDim = v.pDim;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);

    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
      (*this)(i) = v(i);
}

// Destructor
RVector::~RVector() 
{
    delete[] pElem;
}

// Funciones de acceso

// devuelve la dimensin del vector
int RVector::dimension() const
{
    return pDim;
}

// Operadores

// Devuelve el elemento v(i), i=1,2...n
const double& RVector::operator () (int i) const
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}
// Asigna el elemento v(i), i=1,2...n
double& RVector::operator () (int i)
{
    // Comprueba el rango
    if (i < 1 || i > pDim)
	error("Error, indice fuera de rango",CERANGO);
  
    return pElem[i - 1];

}

// Operacin = ( v = v1)  
RVector& RVector::operator = (const RVector& v1)
{
    // Libera la memoria
    delete pElem;
    // Crea un nuevo vector con la dimensin de v  
    pDim = v1.pDim;
    pElem = new double[pDim];
    if (!pElem)
	error("Imposible crear vector",CENOMEM);

    // Y copia cada elemento
    for (int i = 1; i <= pDim; i++)
	(*this)(i) = v1(i);
    return *this;
}

// Operacin += ( v+=v1 ==> v = v +v1)  
RVector& RVector::operator += (const RVector& v1)
{
    // v y v1 deben tener la misma dimensin
    if(pDim != v1.pDim)
	error("Los vectores no tienen la misma dimensin",CEMALDIM);
  
    // Suma los elementos
    for (int i = 1; i <= pDim; i++)
	(*this)(i) += v1(i);
    return *this;
}

// Operacin -= ( v -=v1 ==> v = v - v1)  
RVector& RVector::operator -= (const RVector& v1)
{
    // v y v1 deben tener la misma dimensin
    if(pDim != v1.pDim)
	error("Los vectores no tienen la misma dimensin",CEMALDIM);
  
    // Resta los elementos
    for (int i = 1; i <= pDim; i++)
	(*this)(i) -= v1(i);
    return *this;

}

// Operacin*= ( v*= a --> v = v*a, a es un real)
RVector& RVector::operator *= (double a)
{
    for (int i = 1; i <= pDim; i++)
	(*this)(i) *= a;
    return *this;
}

RVector& RVector::anulaElementos(){
    for (int i = 1; i<= pDim; i++)
	(*this)(i) = 0.;
    return *this;
}

// Lee un vector de un fichero en disco
int RVector::lee(string fichero){
    // Intenta abrir el fichero para lectura
    ifstream fcin(fichero.c_str(),ios::in);
    if ( !fcin){
	error("Imposible abrir fichero en Vector::lee",
	      CENOFCHRO);
	return CENOFCHRO;
    }
    // lee el vector
    for (int i = 1; i <= (*this).dimension(); i++) 
	    fcin >> (*this)(i);
    return EXITO;
}

// Escribe un vector en un fichero en disco
int RVector::escribe(string fichero){
    // Intenta abrir el fichero para escritura
    ofstream fcout(fichero.c_str(),ios::out);
    if ( !fcout){
	error("Imposible abrir fichero en Vector::escribe",
	      CENOFCHRO);
	return CENOFCHRO;
    }
    // escribe el vector
    for (int i = 1; i <= (*this).dimension(); i++)
	fcout << (*this)(i)<< " " ;
    fcout << endl;    
    return EXITO;
}

// Operaciones con vectores
// v= -v1
RVector operator - (const RVector& v1)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    for (int i = 1; i <= v.dimension(); i++)
	v(i) = -v1(i);   
    return v;
}

// v= v1 + v2
RVector operator + (const RVector& v1, const RVector& v2)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v+=v2;            // v = v + v2 --> v1 + v2
    return v;
}

// v= v1 - v2
RVector operator - (const RVector& v1, const RVector& v2)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v-=v2;            // v = v - v2 --> v1 - v2
    return v;
}

// Producto escalar (sobrecarga operador *)
double operator * (const RVector& v1, const RVector& v2)
{
    return productoEscalar(v1,v2);
}
// v = v1*a
RVector operator * (const RVector& v1, double a)
{
    // Crea un nuevo vector como una copia de v1
    RVector v = v1;   // Uso del operador de asignacin (=)
    v*=a;            // v *=a --> v = v*a
    return v;

}
RVector operator * (double a, const RVector& v)
{
    return v*a;  // a* v = v*a
}

double productoEscalar(const RVector& v1, const RVector& v2)
{
    // v1 y v2 deben tener la misma dimensin
    if(v1.dimension() != v2.dimension()) 
	error("Los vectores no tienen la misma dimensin",CEMALDIM);

    double escalar = 0;
    for (int i = 1; i <= v1.dimension(); i++)
	escalar += v1(i)*v2(i);
    return escalar;
}
int maximo(const RVector& v1, int j)
{
    double maximo = 0;
    int imax = 0;
    for (int i=j; i<=v1.dimension();i++){
	if (abs(v1(i)) > maximo){
	    maximo = abs(v1(i));
	    imax = i;
	}
    }
    return imax;    
}
int minimo(const RVector& v1, int j)
{
    double minimo = BIG;  // inicializa a un nmero muy grande
    int imin = 0;
    for (int i=j; i<=v1.dimension();i++){
	if (abs(v1(i)) <= minimo){
	    minimo = abs(v1(i));
	    imin = i;
	}
    }
    return imin;    
}

ostream& operator << (ostream& s, const RVector& v)
{
  s << "( ";
  for (int i = 1; i <= v.dimension(); i++) {
      if (abs(v(i)) < SMALL) 
	  s << setw(10)<<setprecision(2)<< 0<< "\t";
      else
	  s << setw(10)<<setprecision(2)<<v(i) << "\t";
  }
      s << " )";
  return s;
}

