// xlinalg
// Ejemplo de resolucin de problemas
// de lgebra lineal

#include <iostream>
#include <linalg.h>
#include <alineal.h>
#include <error.h>
#include <matutil.h>

void testLU(ALineal& xlu)
{

    // Acciones del algoritmo
    cout << " Matriz LU compacta = " << xlu.lu()<<endl;
    cout << " L= " << xlu.l()<<endl;
    cout << " U= " << xlu.u()<<endl;
    cout << "Permutacin " << xlu.p() << endl;

    // La matriz L est permutadaen general
    // Podemos deshacer las permutaciones
    // multplicando por la matriz inversa

    RMatriz lpu = xlu.l()*xlu.u();
    Permutacion pi = pinv(xlu.p());
    cout << " L*U= " << lpu<<endl;
    cout << " P^(-1)*LU= " << pi*lpu <<endl;
}

void testSistemaLineal(ALineal& xlu, const RMatriz& m, 
		       const RVector& w)
{
    // Resolvemos el sistema
    RVector x = xlu.x();
    cout << " Vector de incgnitas" 
	 << x << endl;

    // Y Comprobamos:
    cout << " m * x - w=" << m*x -w << endl;
}

void testInversa(ALineal& xlu, const RMatriz& m)
{
    // calculamos la inversa y el determinante
    RMatriz minv = xlu.inversa();
    cout << " minv = " << minv << endl;
    cout << " m*minv = " << m*minv << endl;
    cout << " minv*m = " << minv*m << endl;
    cout << "determinante = "<<xlu.det()<<endl;
}

int main ()
{
    // arreglo de strings con los nombres de los ficheros
    // en disco donde guardamos las matrices ejemplo
    string fmatriz[]={"m1d3.dat","m1d4.dat","m1d5.dat"};

    // arreglo de strings con los nombres de los ficheros
    // en disco donde guardamos los vectores de coeficientes
    string fv3[]={"b1d3.dat","b2d3.dat","b3d3.dat"};
    string fv4[]={"b1d4.dat","b2d4.dat","b3d4.dat"};
    string fv5[]={"b1d5.dat","b2d5.dat","b3d5.dat"};

    // arreglo de enteros con la dimensin de las matrices
    int dim[]={3,4,5};

    //Crea un algoritmo de lgebra lineal
    ALineal xal;

    for (int i = 0; i< 2; i++){
	// Crea una matriz de dimensin n
	int n = dim[i];
	RMatriz m(n,n);

	// Y leela del disco
	string fichero = fmatriz[i];
	m.lee(fichero);
	cout << " ejemplo" << i+1 << ": m = " << m << endl;
    
	// Establece la matriz en el algoritmo
	xal.eMatriz(m);
	
	cout << " test LU" << endl;
	testLU(xal);
	cout << " test Inversa" << endl;
	testInversa(xal,m);
	
	// Sistemas lineales
	for (int j = 0; j < 3; j++){
	    // Crea un nuevo vector
	    RVector v(n);
	    // Y leelo del disco
	    if (i ==0) string fichero = fv3[j];
	    else if (i ==1) string fichero = fv4[j];
	    
	    v.lee(fichero);
	    cout << " vector " << j+1 << ": v = " << v << endl;
    
	    // Establece el vector en el algoritmo
	    xal.eVector(v);
	    cout << " test sistema lineal" << endl;
	    testSistemaLineal(xal,m,v);
	}

    }

    // Crea una matriz de dimensin n
    int n = dim[2];
    RMatriz m(n,n);

    // Y leela del disco
    string fichero = fmatriz[2];
    m.lee(fichero);
    cout << " ejemplo 3: m = " << m << endl;
	
    // Sistemas lineales
    for (int j = 0; j < 3; j++){
	// Crea un nuevo vector
	RVector v(n);
	// Y leelo del disco
	string fichero = fv5[j];
	    
	v.lee(fichero);
	cout << " vector " << j+1 << ": v = " << v << endl;
    
	    // Establece la matriz y el vector en el algoritmo
	xal.eSistema(m,v);
	
	cout << " test LU" << endl;
	testLU(xal);
	cout << " test Inversa" << endl;
	testInversa(xal,m);
	cout << " test sistema lineal" << endl;
	testSistemaLineal(xal,m,v);
    }

    return 0;
}
